<script>
  window.MathJax = {
    tex: {
      tags: "ams",
      inlineMath: [['$', '$']],
      processEscapes: true,
      autoload: {
        color: [],
        colorv2: ['color']
      },
      packages: { '[+]': ['noerrors'] }
    },
    chtml: {
      matchFontHeight: false,
      displayAlign: "left",
      displayIndent: "2em"
    },
    options: {
      ignoreHtmlClass: 'tex2jax_ignore',
      processHtmlClass: 'tex2jax_process'
    },
    loader: {
      load: ['input/asciimath', '[tex]/noerrors']
    }
  };
</script>
<script async src="https://cdn.jsdelivr.net/npm/mathjax@3/es5/tex-mml-chtml.js" id="MathJax-script"></script>